@@ -10,10 +10,13 @@ from anyblok.field import FieldException
 from anyblok.column import Column
 from anyblok.mapper import FakeColumn, FakeRelationShip
 from anyblok.relationship import RelationShip, Many2Many
+from anyblok.common import anyblok_column_prefix
 from ..exceptions import SqlBaseException
 from sqlalchemy.orm import aliased, ColumnProperty
 from sqlalchemy.sql.expression import true
-from sqlalchemy import or_, and_
+from sqlalchemy import or_, and_, inspect
+from sqlalchemy_utils.models import NO_VALUE, NOT_LOADED_REPR
+from sqlalchemy.orm.session import object_state
 
 
 class uniquedict(dict):
@@ -29,6 +32,44 @@ class uniquedict(dict):
 
 class SqlMixin:
 
+    def __repr__(self):
+        state = inspect(self)
+        field_reprs = []
+        fields_description = self.fields_description()
+        keys = list(fields_description.keys())
+        keys.sort()
+        for key in keys:
+            type_ = fields_description[key]['type']
+            if key in state.attrs:
+                value = state.attrs.get(key).loaded_value
+            elif (anyblok_column_prefix + key) in state.attrs:
+                value = state.attrs.get(
+                    anyblok_column_prefix + key).loaded_value
+            else:
+                continue
+
+            if value == NO_VALUE:
+                value = NOT_LOADED_REPR
+            elif value and type_ in ('One2Many', 'Many2Many'):
+                value = '<%s len(%d)>' % (
+                    fields_description[key]['model'],
+                    len(value)
+                )
+            elif value and type_ in ('One2One', 'Many2One'):
+                value = '<%s(%s)>' % (
+                    fields_description[key]['model'],
+                    ', '.join(['='.join([x, str(y)])
+                               for x, y in value.to_primary_keys().items()])
+                )
+            else:
+                value = repr(value)
+
+            field_reprs.append('='.join((key, value)))
+
+        return '<%s(%s)>' % (
+            self.__class__.__registry_name__, ', '.join(field_reprs)
+        )
+
     @classmethod
     def define_table_args(cls):
         return ()
@@ -37,6 +78,12 @@ class SqlMixin:
     def define_mapper_args(cls):
         return {}
 
+    @classmethod
+    def get_all_registry_names(cls):
+        models = [base.__registry_name__ for base in cls.__anyblok_bases__]
+        models.insert(0, cls.__registry_name__)
+        return models
+
     @classmethod
     def query(cls, *elements):
         """ Facility to do a SqlAlchemy query::
@@ -140,16 +187,15 @@ class SqlMixin:
         pks = self.get_primary_keys()
         return {x: getattr(self, x) for x in pks}
 
-    @classmethod
+    @classmethod_cache()
     def get_primary_keys(cls):
         """ return the name of the primary keys of the model
 
         :type: list of the primary keys name
         """
         C = cls.registry.System.Column
-        model = cls.__registry_name__
         query = C.query()
-        query = query.filter(C.model == model)
+        query = query.filter(C.model.in_(cls.get_all_registry_names()))
         query = query.filter(C.primary_key == true())
         return query.all().name
 
@@ -320,7 +366,7 @@ class SqlMixin:
 
         return result
 
-    @classmethod
+    @classmethod_cache()
     def getFieldType(cls, name):
         """Return the type of the column
 
@@ -333,12 +379,10 @@ class SqlMixin:
         :param name: name of the column
         :rtype: String, the name of the Type of column used
         """
-        models = [x.__registry_name__ for x in cls.__anyblok_bases__]
-        models.insert(0, cls.__registry_name__)
         Field = cls.registry.System.Field
         query = Field.query()
         query = query.filter(Field.name == name)
-        query = query.filter(Field.model.in_(models))
+        query = query.filter(Field.model.in_(cls.get_all_registry_names()))
         query = query.limit(1)
         return query.one().ftype
 
@@ -359,6 +403,26 @@ class SqlBase(SqlMixin):
     """ this class is inherited by all the SQL model
     """
 
+    def get_modified_fields(self):
+        """return the fields which have changed and their previous values"""
+        state = object_state(self)
+        modified_fields = {}
+        for attr in state.manager.attributes:
+            if not hasattr(attr.impl, 'get_history'):
+                continue
+
+            added, unmodified, deleted = attr.impl.get_history(
+                state, state.dict)
+
+            if added or deleted:
+                field = attr.key
+                if field.startswith(anyblok_column_prefix):
+                    field = field[len(anyblok_column_prefix):]
+
+                modified_fields[field] = deleted[0] if deleted else None
+
+        return modified_fields
+
     def update(self, **values):
         """ Hight livel method to update the session for the instance
         ::
