@@ -58,9 +58,9 @@ class Blok:
         """
         blok = BlokManager.get(self.name)
         path = BlokManager.getPath(self.name)
-        readme = 'README.rst'
-        if hasattr(blok, 'readme'):
-            readme = blok.readme
+        readme = getattr(blok, 'readme', 'README.rst')
+        if readme == '__doc__':
+            return blok.__doc__
 
         file_path = join(path, readme)
         description = ''
@@ -227,9 +227,16 @@ class Blok:
     def load(self):
         """ Method to load the blok when the registry is completly loaded
         """
-        if self.name in BlokManager.bloks:
-            logger.info("Load the blok %r" % self.name)
-            BlokManager.bloks[self.name](self.registry).load()
+        name = self.name
+        blok_cls = BlokManager.get(name)
+        if blok_cls is None:
+            logger.warning("load(): class of Blok %r not found, "
+                           "Blok can't be loaded", name)
+            return
+
+        logger.info("Loading Blok %r", name)
+        blok_cls(self.registry).load()
+        logger.debug("Succesfully loaded Blok %r", name)
 
     @classmethod
     def load_all(cls):
