@@ -1,6 +1,8 @@
 # This file is a part of the AnyBlok project
 #
 #    Copyright (C) 2016 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
+#    Copyright (C) 2017 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
+#    Copyright (C) 2018 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file,You can
@@ -10,20 +12,36 @@ from .mapper import ModelAttributeAdapter
 from sqlalchemy.schema import Sequence as SA_Sequence, Column as SA_Column
 from sqlalchemy import types, CheckConstraint
 from sqlalchemy_utils.types.color import ColorType
-from sqlalchemy_utils.types.encrypted import EncryptedType
+from sqlalchemy_utils.types.encrypted.encrypted_type import EncryptedType
 from sqlalchemy_utils.types.password import PasswordType, Password as SAU_PWD
 from sqlalchemy_utils.types.uuid import UUIDType
 from sqlalchemy_utils.types.url import URLType
+from sqlalchemy_utils.types.phone_number import PhoneNumberType
+from sqlalchemy_utils.types.email import EmailType
+from sqlalchemy_utils.types.scalar_coercible import ScalarCoercible
 from datetime import datetime, date
 from dateutil.parser import parse
-import json
-from copy import deepcopy
 from inspect import ismethod
 from anyblok.config import Configuration
 import time
 import pytz
 import decimal
 from logging import getLogger
+from hashlib import md5
+
+
+pycountry = None
+python_pycountry_type = None
+try:
+    import pycountry
+    if not pycountry.countries._is_loaded:
+        pycountry.countries._load()
+
+    python_pycountry_type = pycountry.countries.data_class
+except ImportError:
+    pass
+
+
 logger = getLogger(__name__)
 
 
@@ -78,6 +96,7 @@ class Column(Field):
     use_hybrid_property = True
     foreign_key = None
     sqlalchemy_type = None
+    type = None
 
     def __init__(self, *args, **kwargs):
         """ Initialize the column
@@ -117,6 +136,13 @@ class Column(Field):
         res['is crypted'] = True if self.encrypt_key else False
         return res
 
+    autodoc_omit_property_values = Field.autodoc_omit_property_values.union((
+        ('foreign_key', None),
+        ('DB column name', None),
+        ('default', None),
+        ('is crypted', False),
+    ))
+
     def native_type(cls):
         """ Return the native SqlAlchemy type """
         return cls.sqlalchemy_type
@@ -228,24 +254,6 @@ class Integer(Column):
     sqlalchemy_type = types.Integer
 
 
-class SmallInteger(Column):
-    """ Small integer column
-
-    ::
-
-        from anyblok.declarations import Declarations
-        from anyblok.column import SmallInteger
-
-
-        @Declarations.register(Declarations.Model)
-        class Test:
-
-            x = SmallInteger(default=1)
-
-    """
-    sqlalchemy_type = types.SmallInteger
-
-
 class BigInteger(Column):
     """ Big integer column
 
@@ -358,16 +366,33 @@ def convert_string_to_datetime(value):
     return value
 
 
+def add_timezone_on_datetime(dt, default_timezone):
+    if dt is not None:
+        if dt.tzinfo is None:
+            dt = default_timezone.localize(dt)
+
+    return dt
+
+
 class DateTimeType(types.TypeDecorator):
 
     impl = types.DateTime(timezone=True)
 
+    def __init__(self, field):
+        self.default_timezone = field.default_timezone
+        self.field = field
+
     def process_bind_param(self, value, engine):
         value = convert_string_to_datetime(value)
-        if value is not None:
-            if value.tzinfo is None:
-                timezone = pytz.timezone(time.tzname[0])
-                value = timezone.localize(value)
+        value = add_timezone_on_datetime(value, self.default_timezone)
+        if self.field.encrypt_key:
+            return value.isoformat()
+
+        return value
+
+    def process_result_value(self, value, dialect):
+        if self.field.encrypt_key:
+            return convert_string_to_datetime(value)
 
         return value
 
@@ -392,20 +417,26 @@ class DateTime(Column):
             x = DateTime(default=datetime.now)
 
     """
-    sqlalchemy_type = DateTimeType
 
     def __init__(self, *args, **kwargs):
         self.auto_update = kwargs.pop('auto_update', False)
+        default_timezone = kwargs.pop(
+            'default_timezone',
+            Configuration.get('default_timezone')
+        )
+        if not default_timezone:
+            default_timezone = time.tzname[0]
+
+        if isinstance(default_timezone, str):
+            default_timezone = pytz.timezone(default_timezone)
+
+        self.default_timezone = default_timezone
+        self.sqlalchemy_type = DateTimeType(self)
         super(DateTime, self).__init__(*args, **kwargs)
 
     def setter_format_value(self, value):
         value = convert_string_to_datetime(value)
-        if value is not None:
-            if value.tzinfo is None:
-                timezone = pytz.timezone(time.tzname[0])
-                value = timezone.localize(value)
-
-        return value
+        return add_timezone_on_datetime(value, self.default_timezone)
 
     def autodoc_get_properties(self):
         res = super(Column, self).autodoc_get_properties()
@@ -461,6 +492,9 @@ class StringType(types.TypeDecorator):
 
         return value
 
+    def process_result_value(self, value, dialect):
+        return value
+
 
 class String(Column):
     """ String column
@@ -548,50 +582,6 @@ class Password(Column):
         return res
 
 
-class uStringType(types.TypeDecorator):
-
-    impl = types.Unicode
-
-    def process_bind_param(self, value, engine):
-        if value is False:
-            value = ''
-
-        return value
-
-
-class uString(Column):
-    """ Unicode column
-
-    ::
-
-        from anyblok.declarations import Declarations
-        from anyblok.column import uString
-
-
-        @Declarations.register(Declarations.Model)
-        class Test:
-
-            x = uString(de", default=u'test')
-
-    """
-    def __init__(self, *args, **kwargs):
-        size = 64
-        if 'size' in kwargs:
-            size = kwargs.pop('size')
-            self.size = size
-
-        if 'type_' in kwargs:
-            del kwargs['type_']
-
-        self.sqlalchemy_type = uStringType(size)
-        super(uString, self).__init__(*args, **kwargs)
-
-    def autodoc_get_properties(self):
-        res = super(uString, self).autodoc_get_properties()
-        res['size'] = self.size
-        return res
-
-
 class TextType(types.TypeDecorator):
 
     impl = types.Text
@@ -602,6 +592,9 @@ class TextType(types.TypeDecorator):
 
         return value
 
+    def process_result_value(self, value, dialect):
+        return value
+
 
 class Text(Column):
     """ Text column
@@ -621,35 +614,6 @@ class Text(Column):
     sqlalchemy_type = TextType
 
 
-class UnicodeTextType(types.TypeDecorator):
-
-    impl = types.UnicodeText
-
-    def process_bind_param(self, value, engine):
-        if value is False:
-            value = ''
-
-        return value
-
-
-class uText(Column):
-    """ Unicode text column
-
-    ::
-
-        from anyblok.declarations import Declarations
-        from anyblok.column import uText
-
-
-        @Declarations.register(Declarations.Model)
-        class Test:
-
-            x = uText(default=u'test')
-
-    """
-    sqlalchemy_type = UnicodeTextType
-
-
 class StrSelection(str):
     """ Class representing the data of one column Selection """
     selections = {}
@@ -713,6 +677,9 @@ class SelectionType(types.TypeDecorator):
 
         return value
 
+    def process_result_value(self, value, dialect):
+        return value
+
 
 class Selection(Column):
     """ Selection column
@@ -817,40 +784,16 @@ class Selection(Column):
             constraint = None
 
         if constraint:
-            name = self.fieldname + '_' + str(len(enum)) + '_types'
+            enum = list(enum)
+            enum.sort()
+            key = md5()
+            key.update(str(enum).encode('utf-8'))
+            name = self.fieldname + '_' + key.hexdigest() + '_types'
             return [CheckConstraint(constraint, name=name)]
 
         return []
 
 
-json_null = object()
-
-
-class JsonType(types.TypeDecorator):
-    """ Generic type for Column JSON """
-    impl = types.Unicode
-
-    def process_bind_param(self, value, dialect):
-        if value is json_null:
-            value = None
-        return json.dumps(value)
-
-    def process_result_value(self, value, dialect):
-        if value is None:
-            return None
-        return json.loads(value)
-
-    def copy_value(self, value):
-        return deepcopy(value)
-
-    def compare_values(self, x, y):
-        return x == y
-
-    @property
-    def python_type(self):
-        return object
-
-
 class Json(Column):
     """ JSON column
 
@@ -866,8 +809,7 @@ class Json(Column):
             x = Json()
 
     """
-    sqlalchemy_type = JsonType
-    Null = json_null
+    sqlalchemy_type = types.JSON(none_as_null=True)
 
 
 class LargeBinary(Column):
@@ -973,9 +915,14 @@ class Color(Column):
 
         return value
 
+    def autodoc_get_properties(self):
+        res = super(Color, self).autodoc_get_properties()
+        res['size'] = self.max_length
+        return res
+
 
 class UUID(Column):
-    """ Sequence column
+    """ UUID column
 
     ::
 
@@ -990,16 +937,25 @@ class UUID(Column):
     """
     def __init__(self, *args, **kwargs):
         uuid_kwargs = {}
+        self.binary = None
+        self.native = None
         for kwarg in ('binary', 'native'):
             if kwarg in kwargs:
                 uuid_kwargs[kwarg] = kwargs.pop(kwarg)
+                setattr(self, kwarg, uuid_kwargs[kwarg])
 
         self.sqlalchemy_type = UUIDType(**uuid_kwargs)
         super(UUID, self).__init__(*args, **kwargs)
 
+    def autodoc_get_properties(self):
+        res = super(UUID, self).autodoc_get_properties()
+        res['binary'] = self.binary
+        res['native'] = self.native
+        return res
+
 
 class URL(Column):
-    """ Integer column
+    """ URL column
 
     ::
 
@@ -1023,3 +979,151 @@ class URL(Column):
                 value = furl(value)
 
         return value
+
+
+class PhoneNumber(Column):
+    """ PhoneNumber column
+
+    ::
+
+        from anyblok.declarations import Declarations
+        from anyblok.column import PhoneNumber
+
+
+        @Declarations.register(Declarations.Model)
+        class Test:
+
+            x = PhoneNumber(default='+120012301')
+
+
+    .. note:: ``phonenumbers`` >= **8.9.5** distribution is required
+
+    """
+    def __init__(self, region='FR', max_length=20, *args, **kwargs):
+        self.region = region
+        self.max_length = max_length
+        if 'type_' in kwargs:
+            del kwargs['type_']
+
+        self.sqlalchemy_type = PhoneNumberType(
+            region=region, max_length=max_length)
+        super(PhoneNumber, self).__init__(*args, **kwargs)
+
+    def setter_format_value(self, value):
+        if isinstance(value, str):
+            value = self.sqlalchemy_type.python_type(value)
+
+        return value
+
+    def autodoc_get_properties(self):
+        res = super(PhoneNumber, self).autodoc_get_properties()
+        res['region'] = self.region
+        res['max_length'] = self.max_length
+        return res
+
+
+class Email(Column):
+    """ Email column
+
+    ::
+
+        from anyblok.column import Email
+
+
+        @Declarations.register(Declarations.Model)
+        class Test:
+
+            x = Email()
+
+    """
+    sqlalchemy_type = EmailType
+
+    def setter_format_value(self, value):
+        if value is not None:
+            return value.lower()
+        return value
+
+
+class CountryType(types.TypeDecorator, ScalarCoercible):
+    """ Generic type for Column Country """
+
+    impl = types.Unicode(3)
+    python_type = python_pycountry_type
+
+    def process_bind_param(self, value, dialect):
+        if value and isinstance(value, self.python_type):
+            return value.alpha_3
+
+        return value
+
+    def process_result_value(self, value, dialect):
+        if value:
+            return pycountry.countries.get(alpha_3=value)
+
+        return value
+
+    def _coerce(self, value):
+        if value is not None and not isinstance(value, self.python_type):
+            return pycountry.countries.get(alpha_3=value)
+
+        return value
+
+
+class Country(Column):
+    """Country column.
+
+    ::
+
+        from anyblok.declarations import Declarations
+        from anyblok.column import Country
+        from pycountry import countries
+
+
+        @Declarations.register(Declarations.Model)
+        class Test:
+
+            x = Country(default=countries.get(alpha_2='FR'))
+
+    """
+    sqlalchemy_type = CountryType
+
+    def __init__(self, mode='alpha_2', *args, **kwargs):
+        self.mode = mode
+        if pycountry is None:
+            raise FieldException(
+                "'pycountry' package is required for use 'CountryType'")
+
+        self.choices = {getattr(country, mode): country.name
+                        for country in pycountry.countries}
+        super(Country, self).__init__(*args, **kwargs)
+
+    def setter_format_value(self, value):
+        if value and not isinstance(value, self.sqlalchemy_type.python_type):
+            try:
+                value = pycountry.countries.get(**{self.mode: value})
+            except LookupError:
+                value = pycountry.countries.lookup(value)
+
+        return value
+
+    def autodoc_get_properties(self):
+        res = super(Country, self).autodoc_get_properties()
+        res['mode'] = self.mode
+        res['choices'] = self.choices
+        return res
+
+    def update_properties(self, registry, namespace, fieldname, properties):
+        super(Country, self).update_properties(registry, namespace,
+                                               fieldname, properties)
+        self.fieldname = fieldname
+        properties['add_in_table_args'].append(self)
+
+    def update_table_args(self, Model):
+        """return check constraint to limit the value"""
+        enum = [country.alpha_3 for country in pycountry.countries]
+        constraint = """"%s" in ('%s')""" % (self.fieldname, "', '".join(enum))
+        enum.sort()
+        key = md5()
+        key.update(str(enum).encode('utf-8'))
+        name = self.fieldname + '_' + key.hexdigest() + '_types'
+        return [CheckConstraint(constraint, name=name)]
