@@ -3,6 +3,7 @@
 #
 #    Copyright (C) 2014 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
 #    Copyright (C) 2017 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
+#    Copyright (C) 2018 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file,You can
@@ -22,6 +23,27 @@ from logging import (getLogger, config, NOTSET, DEBUG, INFO, WARNING, ERROR,
 logger = getLogger(__name__)
 
 
+def get_db_name():
+    """ Return an sqlalchemy name of the database from configuration
+    db_name or db_url
+
+    :rtype: name of the database
+    :exception: ConfigurationException
+    """
+    url = Configuration.get('db_url', None)
+    db_name = Configuration.get('db_name', None)
+
+    if db_name is not None:
+        return db_name
+
+    if url:
+        url = make_url(url)
+        if url.database:
+            return url.database
+
+    raise ConfigurationException("No database's name defined")
+
+
 def get_url(db_name=None):
     """ Return an sqlalchemy URL for database
 
@@ -250,8 +272,7 @@ class Configuration:
 
     This class stores three attributes:
 
-    * groups: lists of options indexed by part, a part is a ``ConfigParser``
-      group, or a process name
+    * groups: lists of options indexed
     * labels: if a group has got a label then all the options in group are
       gathered in a parser group
     * configuration: result of the ``Configuration`` after loading
@@ -269,42 +290,43 @@ class Configuration:
     }
 
     @classmethod
-    def add_configuration_groups(cls, application, new_groups):
-        """Add configuration_groups to an existing application"""
-        if not new_groups:
-            return
+    def add_application_properties(cls, application, new_groups,
+                                   add_default_group=True, **kwargs):
+        """Add argparse properties for an application
 
+        If the application does not exist, the application will be added in
+        the applications properties storage.
+
+        new_groups: extend the existing groups defined. If no group is defined
+        before, this extend the groups of the default application.
+
+        :param application: the name of the application
+        :param new_groups: list of the configuration groups
+        :param add_default_group: if True the default groups will be add
+        :params kwargs: other properties to change
+        """
         if application not in cls.applications:
-            return
-
+            cls.applications[application] = {}
         app = cls.applications[application]
-        if 'configuration_groups' not in app:
-            app['configuration_groups'] = []
-            app['configuration_groups'].extend(
-                cls.applications['default']['configuration_groups'])
+        if kwargs:
+            app.update(kwargs)
 
-        cg = app['configuration_groups']
-        for new_group in new_groups:
-            if new_group not in cg:
-                cg.append(new_group)
+        if new_groups:
+            if 'configuration_groups' not in app:
+                app['configuration_groups'] = []
+                if add_default_group:
+                    app['configuration_groups'].extend(
+                        cls.applications['default']['configuration_groups'])
+
+            cg = app['configuration_groups']
+            for new_group in new_groups:
+                if new_group not in cg:
+                    cg.append(new_group)
 
     @classmethod
-    def init_groups_for(cls, group, part, label):
-        if part not in cls.groups:
-            cls.groups[part] = {group: []}
-        elif group not in cls.groups[part]:
-            cls.groups[part][group] = []
-
-        if label:
-            if part not in cls.labels:
-                cls.labels[part] = {group: label}
-            else:
-                cls.labels[part][group] = label
-
-    @classmethod
-    def add(cls, group, part='bloks', label=None, function_=None,
+    def add(cls, group, label=None, function_=None,
             must_be_loaded_by_unittest=False):
-        """ Add a function in a part and a group.
+        """ Add a function in a group.
 
         The function must have two arguments:
 
@@ -334,7 +356,6 @@ class Configuration:
             def bar(parser, default):
                 pass
 
-        :param part: ConfigParser group or process name
         :param group: group is a set of parser option
         :param label: If the group has a label then all the functions in the
             group are put in group parser
@@ -342,19 +363,24 @@ class Configuration:
         :param must_be_loaded_by_unittest: unittest call this function to init
             configuration of AnyBlok for run unittest"
         """
-        cls.init_groups_for(group, part, label)
+        if group not in cls.groups:
+            cls.groups[group] = []
+
+        if label:
+            cls.labels[group] = label
+
         if function_:
-            if function_ not in cls.groups[part][group]:
+            if function_ not in cls.groups[group]:
                 function_.must_be_loaded_by_unittest = \
                     must_be_loaded_by_unittest
-                cls.groups[part][group].append(function_)
+                cls.groups[group].append(function_)
         else:
 
             def wrapper(function):
-                if function not in cls.groups[part][group]:
+                if function not in cls.groups[group]:
                     function.must_be_loaded_by_unittest = \
                         must_be_loaded_by_unittest
-                    cls.groups[part][group].append(function)
+                    cls.groups[group].append(function)
                 return function
 
             return wrapper
@@ -422,10 +448,10 @@ class Configuration:
     def update(cls, *args, **kwargs):
         if args:
             if len(args) > 1:
-                raise ConfigurationException("Wainting only one dict")
+                raise ConfigurationException("Waiting only one dict")
 
             if not isinstance(args[0], dict):
-                raise ConfigurationException("Wainting a dict")
+                raise ConfigurationException("Waiting a dict")
 
             for k, v in args[0].items():
                 cls.set(k, v)
@@ -438,7 +464,7 @@ class Configuration:
         cls.configuration[key] = ConfigOption(value, type)
 
     @classmethod
-    def remove_label(cls, group, part='bloks'):
+    def remove_label(cls, group):
         """ Remove an existing label
 
         The goal of this function is to remove an existing label of a specific
@@ -450,15 +476,13 @@ class Configuration:
 
             Configuration.remove_label('create-db')
 
-        :param part: ConfigParser group or process name
         :param group: group is a set of parser option
         """
-        if part in cls.labels:
-            if group in cls.labels[part]:
-                del cls.labels[part][group]
+        if group in cls.labels:
+            del cls.labels[group]
 
     @classmethod
-    def remove(cls, group, function_, part='bloks'):
+    def remove(cls, group, function_):
         """ Remove an existing function
 
         If your application inherits some unwanted options from a specific
@@ -470,59 +494,12 @@ class Configuration:
             Configuration.add('create-db', function_=foo)
             Configuration.remove('create-db', function_=foo)
 
-        :param part: ConfigParser group or process name
         :param group: group is a set of parser option
         :param function_: function to add
         """
-        if part in cls.groups:
-            if group in cls.groups[part]:
-                if function_ in cls.groups[part][group]:
-                    cls.groups[part][group].remove(function_)
-
-    @classmethod
-    def _merge_groups(cls, *parts):
-        """ Internal method to merge groups in function of parts
-
-        :param parts: parts to merge
-        :exception: ConfigurationException
-        """
-        if not parts:
-            raise ConfigurationException("No parts to merge")
-
-        groups = {}
-
-        for part in parts:
-            if part not in cls.groups:
-                continue
-
-            for k, v in cls.groups[part].items():
-                if k in groups:
-                    groups[k].extend(v)
-                else:
-                    groups[k] = [] + v
-
-        return groups
-
-    @classmethod
-    def _merge_labels(cls, *parts):
-        """ Internal method to merge labels in function of parts
-
-        :param parts: parts to merge
-        :exception: ConfigurationException
-        """
-        if not parts:
-            raise ConfigurationException("No parts to merge")
-
-        labels = {}
-
-        for part in parts:
-            if part not in cls.labels:
-                continue
-
-            for k, v in cls.labels[part].items():
-                labels[k] = v
-
-        return labels
+        if group in cls.groups:
+            if function_ in cls.groups[group]:
+                cls.groups[group].remove(function_)
 
     @classmethod
     @log(logger)
@@ -530,24 +507,20 @@ class Configuration:
         """Load the argparse configuration need for the unittest"""
         if not cls.configuration:
             parser = getParser()
-            for parts in cls.groups.values():
-                for part, fncts in parts.items():
-                    for fnct in fncts:
-                        if (
-                            fnct.must_be_loaded_by_unittest or
-                            part in ('plugins',)
-                        ):
-                            fnct(parser)
+            for group in cls.groups.keys():
+                for fnct in cls.groups[group]:
+                    if (
+                        fnct.must_be_loaded_by_unittest or
+                        group in ('plugins',)
+                    ):
+                        fnct(parser)
 
     @classmethod
     @log(logger, level='debug')
-    def load(cls, application, configuration_groups=None,
-             parts_to_load=('bloks',), useseparator=False, **kwargs):
+    def load(cls, application, useseparator=False, **kwargs):
         """ Load the argparse definition and parse them
 
         :param application: name of the application
-        :param configuration_groups: iterable configuration group to load
-        :param parts_to_load: group of blok to load
         :param useseparator: boolean(default False)
         :param \**kwargs: ArgumentParser named arguments
         """
@@ -561,12 +534,12 @@ class Configuration:
             description.update(cls.applications['default'])
 
         description.update(kwargs)
-        _configuration_groups = description.pop(
+        configuration_groups = description.pop(
             'configuration_groups',
             cls.applications['default']['configuration_groups'])
-        configuration_groups = set(configuration_groups or []).union(
-            _configuration_groups)
-        configuration_groups.add('plugins')
+        if 'plugins' not in configuration_groups:
+            configuration_groups.append('plugins')
+
         if useseparator:
             parser = getParser(**description)
 
@@ -578,39 +551,35 @@ class Configuration:
         else:
             parser = getParser(**description)
 
-        cls._load(parser, configuration_groups, parts_to_load)
+        cls._load(parser, configuration_groups)
         args = parser.parse_args(our_argv)
         if sep is not None:
             del sys.argv[1:sep+1]
 
-        cls.parse_options(args, parts_to_load)
+        cls.parse_options(args)
 
     @classmethod
-    def _load(cls, parser, configuration_groups, parts_to_load):
+    def _load(cls, parser, configuration_groups):
         """ Load the argparse definition and parse them
 
         :param description: description of configuration
         :param configuration_groups: list configuration groupe to load
-        :param parts_to_load: group of blok to load
         :param useseparator: boolean(default False)
         """
         if configuration_groups is None:
             return
 
-        groups = cls._merge_groups(*parts_to_load)
-        labels = cls._merge_labels(*parts_to_load)
-
-        for group in groups:
+        for group in cls.groups:
             if group not in configuration_groups:
                 continue
 
-            label = labels.get(group)
+            label = cls.labels.get(group)
             if label:
                 g = parser.add_argument_group(label)
             else:
                 g = parser
 
-            for function in groups[group]:
+            for function in cls.groups[group]:
                 function(g)
 
     @classmethod
@@ -653,11 +622,15 @@ class Configuration:
                 getLogger(qualname).setLevel(level)
 
     @classmethod
-    def parse_configfile(cls, configfile, parts_to_load):
+    def parse_configfile(cls, configfile, required):
         cur_cwd = os.getcwd()
         configfile = os.path.abspath(configfile)
-        print('Load config file %r' % configfile)
+        print('Loading config file %r' % configfile)
         if not isfile(configfile):
+            if required:
+                raise ConfigurationException(
+                    "No such file or not a regular file: %r " % configfile)
+
             return
 
         cwd_file, file_name = os.path.split(configfile)
@@ -666,9 +639,7 @@ class Configuration:
             os.chdir(cwd_file)
             cparser = ConfigParser()
             cparser.read(configfile)
-            sections = [y for x in parts_to_load + ('AnyBlok',)
-                        if cparser.has_section(x)
-                        for y in cparser.items(x)]
+            sections = cparser.items('AnyBlok')
 
             for opt, value in sections:
                 if opt in ('logging_configfile', 'json_logging_configfile',
@@ -681,7 +652,7 @@ class Configuration:
             if 'extend' in configuration:
                 extend = configuration.pop('extend')
                 if extend:
-                    cls.parse_configfile(extend, parts_to_load)
+                    cls.parse_configfile(extend, True)
 
         finally:
             os.chdir(cur_cwd)
@@ -689,7 +660,7 @@ class Configuration:
         cls.update(**configuration)
 
     @classmethod
-    def parse_options(cls, arguments, parts_to_load):
+    def parse_options(cls, arguments):
         if arguments._get_args():
             raise ConfigurationException(
                 'Positional arguments are forbidden')
@@ -697,13 +668,13 @@ class Configuration:
         ad = AppDirs('AnyBlok')
         # load the global configuration file
         cls.parse_configfile(
-            join(ad.site_config_dir, 'conf.cfg'), parts_to_load)
+            join(ad.site_config_dir, 'conf.cfg'), False)
         # load the user configuration file
         cls.parse_configfile(
-            join(ad.user_config_dir, 'conf.cfg'), parts_to_load)
+            join(ad.user_config_dir, 'conf.cfg'), False)
         if 'configfile' in dict(arguments._get_kwargs()).keys():
             if arguments.configfile:
-                cls.parse_configfile(arguments.configfile, parts_to_load)
+                cls.parse_configfile(arguments.configfile, True)
 
         for opt, value in arguments._get_kwargs():
             if opt not in cls.configuration or value:
@@ -735,6 +706,15 @@ def add_configuration_file(parser):
                         help="Relative path of the config file")
     parser.add_argument('--without-auto-migration', dest='withoutautomigration',
                         action='store_true')
+    parser.add_argument('--isolation-level',
+                        default="READ_COMMITTED",
+                        choices=["SERIALIZABLE", "REPEATABLE_READ",
+                                 "READ_COMMITTED", "READ_UNCOMMITTED",
+                                 "AUTOCOMMIT"])
+    parser.add_argument('--default-timezone',
+                        default=os.environ.get('ANYBLOK_DEFAULT_TIMEZONE'),
+                        help="default timezone use by naive datetime "
+                             "(by default use the timezone of the serveur")
 
 
 @Configuration.add('database', label="Database",
