@@ -112,6 +112,24 @@ def get_fields(base, without_relationship=False, only_relationship=False,
     return fields
 
 
+def autodoc_fields(declaration_cls, model_cls):
+    """Produces autodocumentation table for the fields.
+
+    Exposed as a function in order to be reusable by a simple export,
+    e.g., from anyblok.mixin.
+    """
+    if not has_sql_fields([model_cls]):
+        return ''
+
+    rows = [['Fields', '']]
+    rows.extend([x, y.autodoc()]
+                for x, y in get_fields(model_cls).items())
+    table = Texttable(max_width=0)
+    table.set_cols_valign(["m", "t"])
+    table.add_rows(rows)
+    return table.draw() + '\n\n'
+
+
 @Declarations.add_declaration_type(isAnEntry=True,
                                    pre_assemble='pre_assemble_callback',
                                    assemble='assemble_callback',
@@ -151,6 +169,12 @@ class Model:
         the table. But they must have the same column.
     """
 
+    autodoc_anyblok_kwargs = True
+
+    autodoc_anyblok_bases = True
+
+    autodoc_anyblok_fields = True
+
     @classmethod
     def pre_assemble_callback(cls, registry):
         plugins = get_model_plugins(registry)
@@ -613,22 +637,3 @@ class Model:
         bloks = Blok.list_by_state('touninstall')
         Blok.uninstall_all(*bloks)
         return Blok.apply_state(*registry.ordered_loaded_bloks)
-
-    @classmethod
-    def autodoc_class(cls, model_cls):
-        res = [":Declaration type: Model"]
-        res.extend([':%s: %s' % (x.replace('_', ' ').strip(), str(y))
-                    for x, y in model_cls.__anyblok_kwargs__.items()])
-        res.extend([':Inherit model or mixin:', ''])
-        res.extend([' * ' + str(x) for x in model_cls.__anyblok_bases__])
-        res.extend(['', ''])
-        if has_sql_fields([model_cls]):
-            rows = [['field name', 'Description']]
-            rows.extend([x, y.autodoc()]
-                        for x, y in get_fields(model_cls).items())
-            table = Texttable()
-            table.set_cols_valign(["m", "t"])
-            table.add_rows(rows)
-            res.extend(['', table.draw(), '', ''])
-
-        return '\n'.join(res)
