@@ -6,7 +6,8 @@
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file,You can
 # obtain one at http://mozilla.org/MPL/2.0/.
-from os.path import join, exists
+from os.path import join
+from os import walk
 from logging import getLogger
 import nose
 
@@ -39,6 +40,13 @@ class RegistryConflictingException(Exception):
     """ Simple Exception for Registry """
 
 
+def return_list(entry):
+    if entry is not None and not isinstance(entry, (list, tuple)):
+        entry = [entry]
+
+    return entry
+
+
 class RegistryManager:
     """ Manage the global registry
 
@@ -99,23 +107,31 @@ class RegistryManager:
             unload_callback()
 
     @classmethod
-    def get(cls, db_name, loadwithoutmigration=False, **kwargs):
+    def get(cls, db_name, loadwithoutmigration=False, log_repeat=True,
+            **kwargs):
         """ Return an existing Registry
 
         If the Registry doesn't exist then the Registry are created and added
         to registries dict
 
         :param db_name: the name of the database linked to this registry
+        :param loadwithoutmigration: if True, registry is created without
+                                     any migration of the database
+        :param log_repeat: if False, when the registry is load whitout
+                           migration, the warning is not logged
         :rtype: ``Registry``
         """
         EnvironmentManager.set('db_name', db_name)
         if db_name in cls.registries:
-            if loadwithoutmigration:
-                logger.warning("loadwithoutmigration can not be used because "
-                               "the registry for %r is already load" % db_name)
+            if loadwithoutmigration and log_repeat:
+                logger.warning(
+                    "Ignoring loadwithoutmigration=True for database %r "
+                    "because its registry is already loaded", db_name)
             return cls.registries[db_name]
 
         _Registry = Configuration.get('Registry', Registry)
+        logger.info("Loading registry for database %r with class %r",
+                    db_name, _Registry)
         registry = _Registry(
             db_name, loadwithoutmigration=loadwithoutmigration, **kwargs)
         cls.registries[db_name] = registry
@@ -443,6 +459,10 @@ class Registry:
             max_overflow=Configuration.get('db_max_overflow') or 10,
             echo_pool=Configuration.get('db_echo_pool') or False,
             pool_size=Configuration.get('db_pool_size') or 5,
+            isolation_level=self.additional_setting.get(
+                'isolation_level',
+                Configuration.get('isolation_level', 'READ_UNCOMMITTED')
+            ),
         )
 
     def init_engine(self, db_name=None):
@@ -941,9 +961,9 @@ class Registry:
 
         test_blok = blok2install and Configuration.get(
             'test_blok_at_install')
-        selected_bloks = Configuration.get('selected_bloks')
+        selected_bloks = return_list(Configuration.get('selected_bloks'))
         in_selected_bloks = blok2install in (selected_bloks or [blok2install])
-        unwanted_bloks = Configuration.get('unwanted_bloks')
+        unwanted_bloks = return_list(Configuration.get('unwanted_bloks'))
         not_in_unwanted_bloks = blok2install not in (unwanted_bloks or [])
 
         if test_blok and in_selected_bloks and not_in_unwanted_bloks:
@@ -977,30 +997,29 @@ class Registry:
             funct(self.session)
 
     def run_test(registry, blok2install):
-        defaultTest = join(BlokManager.getPath(blok2install), 'tests')
-        if exists(defaultTest):
+        startpath = BlokManager.getPath(blok2install)
 
-            class ContextSuite(nose.suite.ContextSuite):
+        class ContextSuite(nose.suite.ContextSuite):
 
-                def __init__(self, *args, **kwargs):
-                    super(ContextSuite, self).__init__(*args, **kwargs)
-                    if self.context is not None:
-                        self.context.registry = registry
+            def __init__(self, *args, **kwargs):
+                super(ContextSuite, self).__init__(*args, **kwargs)
+                if self.context is not None:
+                    self.context.registry = registry
 
-            class ContextSuiteFactory(nose.suite.ContextSuiteFactory):
-                suiteClass = ContextSuite
+        class ContextSuiteFactory(nose.suite.ContextSuiteFactory):
+            suiteClass = ContextSuite
 
-            class TestLoader(nose.loader.TestLoader):
+        class TestLoader(nose.loader.TestLoader):
 
-                def __init__(self, *args, **kwargs):
-                    super(TestLoader, self).__init__(*args, **kwargs)
-                    self.suiteClass = ContextSuiteFactory(config=self.config)
+            def __init__(self, *args, **kwargs):
+                super(TestLoader, self).__init__(*args, **kwargs)
+                self.suiteClass = ContextSuiteFactory(config=self.config)
 
-            nose.run(defaultTest=[defaultTest], testLoader=TestLoader(),
-                     argv=['-v', '-s'])
-        else:
-            logger.warning("Blok %r has no %r directory" % (
-                blok2install, defaultTest))
+        for root, dirs, _ in walk(startpath):
+            if 'tests' in dirs:
+                nose.run(defaultTest=[join(root, 'tests')],
+                         testLoader=TestLoader(),
+                         argv=['-v', '-s'])
 
     def assemble_entries(self):
         for entry in RegistryManager.declared_entries:
@@ -1082,8 +1101,9 @@ class Registry:
         self.session.expire(obj, attribute_names=attribute_names)
 
     def expire_all(self):
-        """Expire all the objects in session::
+        """Expire all the objects in session
 
+        ::
             registry.expire_all()
 
         """
@@ -1116,7 +1136,7 @@ class Registry:
         self.session.refresh(obj, attribute_names=attribute_names)
 
     def rollback(self, *args, **kwargs):
-        logger.debug('[ROLLACK] with args=%r and kwargs = %r', args, kwargs)
+        logger.debug('[ROLLBACK] with args=%r and kwargs = %r', args, kwargs)
         self.session.rollback(*args, **kwargs)
         EnvironmentManager.set('_precommit_hook', [])
         EnvironmentManager.set('_postcommit_hook', [])
