@@ -13,6 +13,7 @@ from sqlalchemy.schema import Column as SA_Column
 from sqlalchemy.ext.declarative import declared_attr
 from sqlalchemy.ext.hybrid import hybrid_property
 from sqlalchemy import exc as sa_exc, util
+from sqlalchemy_utils.functions import get_class_by_table
 from .field import Field, FieldException
 from .mapper import ModelAdapter, ModelAttribute, ModelRepr
 from anyblok.common import anyblok_column_prefix
@@ -306,6 +307,8 @@ class Many2One(RelationShip):
                                                  "foreigh key",
                                     nullable=True,
                                     unique=False,
+                                    index=False,
+                                    primary_key=False,
                                     one2many="themodels")
 
     If the ``remote_columns`` are not define then, the system takes the primary
@@ -319,7 +322,9 @@ class Many2One(RelationShip):
     :param remote_columns: the column name on the remote model
     :param column_names: the column on the model which have the foreign key
     :param nullable: If the column_names is nullable
-    :param unique: If True, add the unique constraint on the column
+    :param unique: If True, add the unique constraint on the columns
+    :param index: If True, add the index constraint on the columns
+    :param primary_key: If True, add the primary_key=True on the columns
     :param one2many: create the one2many link with this many2one
     """
     use_hybrid_property = True
@@ -338,10 +343,14 @@ class Many2One(RelationShip):
             self.nullable = self.kwargs.pop('nullable')
             self.kwargs['info']['nullable'] = self.nullable
 
-        self.unique = False
-        if 'unique' in kwargs:
-            self.unique = self.kwargs.pop('unique')
-            self.kwargs['info']['unique'] = self.unique
+        self.unique = self.kwargs.pop('unique', False)
+        self.kwargs['info']['unique'] = self.unique
+
+        self.index = self.kwargs.pop('index', False)
+        self.kwargs['info']['index'] = self.index
+
+        self.primary_key = self.kwargs.pop('primary_key', False)
+        self.kwargs['info']['primary_key'] = self.primary_key
 
         if 'one2many' in kwargs:
             self.kwargs['backref'] = self.kwargs.pop('one2many')
@@ -358,11 +367,20 @@ class Many2One(RelationShip):
 
     def autodoc_get_properties(self):
         res = super(Many2One, self).autodoc_get_properties()
-        res['_remote_columns'] = self._remote_columns
-        res['_column_names'] = self._column_names
+        res['remote_columns'] = self._remote_columns
+        res['column_names'] = self._column_names
         res['unique'] = self.unique
+        res['index'] = self.index
+        res['primary_key'] = self.primary_key
         return res
 
+    autodoc_omit_property_values = Field.autodoc_omit_property_values.union((
+        ('remote_columns', None),
+        ('column_names', None),
+        ('unique', False),
+        ('primary_key', False),
+    ))
+
     def get_remote_columns(self, registry):
         if self._remote_columns is None:
             return self.model.primary_keys(registry)
@@ -456,7 +474,7 @@ class Many2One(RelationShip):
                                        cname.attribute_name)
             if not cname.is_declared(registry):
                 rc, remote_type = self.get_column_information(
-                    registry, cname, remote_types)
+                    registry, cname, remote_types, fieldname)
                 cname.add_fake_column(registry)
                 foreign_key = remote_columns[rc].get_fk_name(registry)
                 self.create_column(cname, remote_type, foreign_key, properties)
@@ -487,15 +505,14 @@ class Many2One(RelationShip):
                                  **self.foreign_key_options)
         ]
 
-    def get_column_information(self, registry, cname, remote_types):
+    def get_column_information(self, registry, cname, remote_types, fieldname):
         if len(remote_types) == 1:
             rc = [x for x in remote_types][0]
             return rc, remote_types[rc]
         else:
             rc = cname.get_fk_column(registry)
             if rc is None:
-                rc = cname.attribute_name[len(
-                    self.model.tablename(registry)) + 1:]
+                rc = cname.attribute_name[len(fieldname) + 1:]
 
             if rc in remote_types:
                 return rc, remote_types[rc]
@@ -529,6 +546,8 @@ class Many2One(RelationShip):
                 remote_type,
                 nullable=self.nullable,
                 unique=self.unique,
+                index=self.index,
+                primary_key=self.primary_key,
                 info=dict(label=self.label, foreign_key=foreign_key))
 
         properties[(anyblok_column_prefix +
@@ -700,6 +719,7 @@ class Many2Many(RelationShip):
 
     :param model: the remote model
     :param join_table: the many2many table to join local and remote models
+    :param join_model: rich many2many where the join table come from a Model
     :param remote_columns: the column name on the remote model
     :param m2m_remote_columns: the column name to remote model in m2m table
     :param local_columns: the column on the model
@@ -711,6 +731,10 @@ class Many2Many(RelationShip):
         super(Many2Many, self).__init__(**kwargs)
 
         self.join_table = self.kwargs.pop('join_table', None)
+        self.join_model = self.kwargs.pop('join_model', None)
+        if self.join_model:
+            self.join_model = ModelAdapter(self.join_model)
+
         self.remote_columns = self.kwargs.pop('remote_columns', None)
         if self.remote_columns and not isinstance(self.remote_columns,
                                                   (list, tuple)):
@@ -731,34 +755,79 @@ class Many2Many(RelationShip):
                                                      (list, tuple)):
             self.m2m_local_columns = [self.m2m_local_columns]
 
+        self.compute_join = self.kwargs.pop('compute_join', False)
         self.kwargs['backref'] = self.kwargs.pop('many2many', None)
         self.kwargs['info']['remote_name'] = self.kwargs['backref']
 
     def autodoc_get_properties(self):
         res = super(Many2Many, self).autodoc_get_properties()
-        res['join table'] = self.join_table
+        if self.join_table:
+            res['join table'] = self.join_table
+
+        if self.join_model:
+            res['join model'] = self.join_model.model_name
+
         res['remote_columns'] = self.remote_columns
         res['m2m_remote_columns'] = self.m2m_remote_columns
         res['local_columns'] = self.local_columns
         res['m2m_local_columns'] = self.m2m_local_columns
+        res['compute_join'] = self.compute_join
         return res
 
-    def get_m2m_columns(self, registry, columns, m2m_columns):
+    def get_m2m_columns(self, registry, columns, m2m_columns, modelname,
+                        suffix=""):
         if m2m_columns is None:
-            m2m_columns = [x.get_fk_name(registry).replace('.', '_')
-                           for x in columns]
+            m2m_columns = [
+                x.get_fk_name(registry).replace('.', '_') + suffix
+                for x in columns]
+        elif self.join_model:
+            m2m_columns_ = []
+            first_step = registry.loaded_namespaces_first_step[
+                self.join_model.model_name]
+            for col in m2m_columns:
+                if col not in first_step:
+                    m2m_columns_.append(col)
+                elif isinstance(first_step[col], (Many2One, One2One)):
+                    c = first_step[col]
+                    remote_columns = c.get_remote_columns(registry)
+                    m2m_columns_.extend([
+                        x.attribute_name
+                        for x in c.get_columns_names(
+                            registry,
+                            self.join_model.model_name,
+                            col,
+                            remote_columns
+                        )
+                    ])
+                else:
+                    m2m_columns_.append(col)
+
+            m2m_columns = m2m_columns_
+
+        if len(columns) != len(m2m_columns):
+            raise FieldException((
+                "The number of the column (%r) is not the same that the "
+                "number m2m column (%r)") % (
+                    columns, m2m_columns
+                )
+            )
 
         cols = []
         col_names = []
         ref_cols = []
+        primaryjoin = []
         for i, column in enumerate(m2m_columns):
             sqltype = columns[i].native_type(registry)
             foreignkey = columns[i].get_fk_name(registry)
+            completename = columns[i].get_complete_name(registry)
             cols.append(Column(column, sqltype, primary_key=True))
             col_names.append(column)
             ref_cols.append(foreignkey)
+            primaryjoin.append(
+                modelname + '.' + column + ' == ' + completename)
 
-        return cols, ForeignKeyConstraint(col_names, ref_cols)
+        primaryjoin = 'and_(' + ', '.join(primaryjoin) + ')'
+        return cols, ForeignKeyConstraint(col_names, ref_cols), primaryjoin
 
     def get_local_and_remote_columns(self, registry):
         if not self.local_columns:
@@ -780,6 +849,38 @@ class Many2Many(RelationShip):
 
         return local_columns, remote_columns
 
+    def get_join_table(self, registry, namespace, fieldname):
+        """Get the join table name from join_table or join_model
+
+        :param registry: the registry which load the relationship
+        :param namespace: the name space of the model
+        :param fieldname: fieldname of the relationship
+        :rtype: name of the join table
+        :exception: FieldException
+        """
+        join_table = self.join_table
+        join_model_table = None
+        if self.join_model:
+            join_model_table = self.join_model.tablename(registry)
+
+        if join_table is None and join_model_table is None:
+            join_table = 'join_%s_and_%s' % (
+                self.local_model.tablename(registry),
+                self.model.tablename(registry))
+
+        elif join_table and join_model_table and join_table != join_model_table:
+            raise FieldException(
+                (
+                    "The join_table %r and join_model %r is both declared, "
+                    "on model %r and many2many %r, "
+                    "but the both table name are different and we can not "
+                    "determinate which is the good table's name"
+                ) % (self.join_table, self.join_model.model_name,
+                     namespace, fieldname)
+            )
+
+        return join_table or join_model_table
+
     def get_sqlalchemy_mapping(self, registry, namespace, fieldname,
                                properties):
         """ Create the relationship
@@ -794,21 +895,62 @@ class Many2Many(RelationShip):
         self.local_model = ModelRepr(namespace)
         local_columns, remote_columns = self.get_local_and_remote_columns(
             registry)
-        join_table = self.join_table
-        if self.join_table is None:
-            join_table = 'join_%s_and_%s' % (
-                self.local_model.tablename(registry),
-                self.model.tablename(registry))
-
+        join_table = self.get_join_table(registry, namespace, fieldname)
         if join_table not in registry.declarativebase.metadata.tables:
-            remote_columns, remote_fk = self.get_m2m_columns(
-                registry, remote_columns, self.m2m_remote_columns)
-            local_columns, local_fk = self.get_m2m_columns(
-                registry, local_columns, self.m2m_local_columns)
+            modelname = ''.join(x.capitalize() for x in join_table.split('_'))
+            remote_columns, remote_fk, secondaryjoin = self.get_m2m_columns(
+                registry, remote_columns, self.m2m_remote_columns, modelname,
+                suffix="right" if namespace == self.model.model_name else ""
+            )
+            local_columns, local_fk, primaryjoin = self.get_m2m_columns(
+                registry, local_columns, self.m2m_local_columns, modelname,
+                suffix="left" if namespace == self.model.model_name else ""
+            )
 
-            Table(join_table, registry.declarativebase.metadata, *(
+            Node = Table(join_table, registry.declarativebase.metadata, *(
                 local_columns + remote_columns + [local_fk, remote_fk]))
 
+            if namespace == self.model.model_name:
+                type(modelname, (registry.declarativebase,), {
+                    '__table__': Node
+                })
+                self.kwargs['primaryjoin'] = primaryjoin
+                self.kwargs['secondaryjoin'] = secondaryjoin
+
+        elif namespace == self.model.model_name or self.compute_join:
+            table = registry.declarativebase.metadata.tables[join_table]
+            cls = get_class_by_table(registry.declarativebase, table)
+            modelname = ModelRepr(cls.__registry_name__).modelname(registry)
+            if (
+                self.m2m_local_columns is None and
+                self.m2m_remote_columns is None
+            ):
+                raise FieldException(
+                    "No 'm2m_local_columns' and 'm2m_remote_columns' "
+                    "attribute filled for many2many "
+                    "%r on model %r" % (fieldname, namespace))
+            elif self.m2m_local_columns is None:
+                raise FieldException(
+                    "No 'm2m_local_columns' attribute filled for many2many "
+                    "%r on model %r" % (fieldname, namespace))
+            elif self.m2m_remote_columns is None:
+                raise FieldException(
+                    "No 'm2m_remote_columns' attribute filled for many2many"
+                    " %r on model %r" % (fieldname, namespace))
+
+            remote_columns, remote_fk, secondaryjoin = self.get_m2m_columns(
+                registry, remote_columns, self.m2m_remote_columns,
+                modelname,
+                suffix="right" if namespace == self.model.model_name else ""
+            )
+
+            local_columns, local_fk, primaryjoin = self.get_m2m_columns(
+                registry, local_columns, self.m2m_local_columns, modelname,
+                suffix="left" if namespace == self.model.model_name else ""
+            )
+            self.kwargs['primaryjoin'] = primaryjoin
+            self.kwargs['secondaryjoin'] = secondaryjoin
+
         self.kwargs['secondary'] = join_table
         # definition of expiration
         if self.kwargs.get('backref'):
