@@ -1,20 +1,21 @@
 # This file is a part of the AnyBlok project
 #
 #    Copyright (C) 2014 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
+#    Copyright (C) 2017 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
+#    Copyright (C) 2018 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file,You can
 # obtain one at http://mozilla.org/MPL/2.0/.
 from anyblok.tests.testcase import TestCase, DBTestCase
-from sqlalchemy import Integer as SA_Integer
+from sqlalchemy import Integer as SA_Integer, String as SA_String
 from sqlalchemy.exc import StatementError
 from anyblok import Declarations
 from anyblok.field import FieldException
-from anyblok.column import (Column, Boolean, Json, String, BigInteger,
-                            SmallInteger, uString, Text, uText, Selection,
-                            Date, DateTime, Time, Interval, Decimal, Float,
-                            LargeBinary, Integer, Sequence, Color, Password,
-                            UUID, URL)
+from anyblok.column import (
+    Column, Boolean, Json, String, BigInteger, Text, Selection, Date, DateTime,
+    Time, Interval, Decimal, Float, LargeBinary, Integer, Sequence, Color,
+    Password, UUID, URL, PhoneNumber, Email, Country)
 from unittest import skipIf
 
 try:
@@ -42,6 +43,20 @@ except Exception:
     has_furl = False
 
 
+try:
+    import phonenumbers  # noqa
+    has_phonenumbers = True
+except Exception:
+    has_phonenumbers = False
+
+try:
+    import pycountry  # noqa
+    has_pycountry = True
+    from sqlalchemy_utils import PhoneNumber as PN
+except Exception:
+    has_pycountry = False
+
+
 Model = Declarations.Model
 register = Declarations.register
 
@@ -124,11 +139,6 @@ class TestColumns(DBTestCase):
         test = registry.Test.insert(col=1)
         self.assertEqual(test.col, 1)
 
-    def test_small_integer(self):
-        registry = self.init_registry(simple_column, ColumnType=SmallInteger)
-        test = registry.Test.insert(col=1)
-        self.assertEqual(test.col, 1)
-
     def test_Float(self):
         registry = self.init_registry(simple_column, ColumnType=Float)
         test = registry.Test.insert(col=1.0)
@@ -195,11 +205,26 @@ class TestColumns(DBTestCase):
         registry = self.init_registry(simple_column, ColumnType=String,
                                       encrypt_key='secretkey')
         test = registry.Test.insert(col='col')
+        registry.session.commit()
         self.assertEqual(test.col, 'col')
         res = registry.execute('select col from test where id = %s' % test.id)
         res = res.fetchall()[0][0]
         self.assertNotEqual(res, 'col')
 
+    @skipIf(not has_cryptography, "cryptography is not installed")
+    def test_datetime_with_encrypt_key(self):
+        import datetime
+        import time
+        import pytz
+
+        timezone = pytz.timezone(time.tzname[0])
+        now = datetime.datetime.now().replace(tzinfo=timezone)
+        registry = self.init_registry(simple_column, ColumnType=DateTime,
+                                      encrypt_key='secretkey')
+        test = registry.Test.insert(col=now)
+        registry.session.commit()
+        self.assertEqual(test.col, now)
+
     def test_string_with_size(self):
         registry = self.init_registry(
             simple_column, ColumnType=String, size=100)
@@ -245,72 +270,6 @@ class TestColumns(DBTestCase):
         self.registry.expire(test, ['col'])
         self.assertEqual(test.col, '')
 
-    def test_ustring(self):
-        registry = self.init_registry(simple_column, ColumnType=uString)
-        test = registry.Test.insert(col='col')
-        self.assertEqual(test.col, 'col')
-
-    def test_ustring_with_size(self):
-        registry = self.init_registry(
-            simple_column, ColumnType=uString, size=100)
-        test = registry.Test.insert(col='col')
-        self.assertEqual(test.col, 'col')
-
-    def test_ustring_with_False(self):
-        registry = self.init_registry(simple_column, ColumnType=uString)
-        test = registry.Test.insert(col=False)
-        self.assertEqual(test.col, False)
-        self.registry.flush()
-        self.registry.expire(test, ['col'])
-        self.assertEqual(test.col, '')
-
-    def test_ustring_set_False(self):
-        registry = self.init_registry(simple_column, ColumnType=uString)
-        test = registry.Test.insert()
-        test.col = False
-        self.assertEqual(test.col, False)
-        self.registry.flush()
-        self.registry.expire(test, ['col'])
-        self.assertEqual(test.col, '')
-
-    def test_ustring_query_False(self):
-        registry = self.init_registry(simple_column, ColumnType=uString)
-        test = registry.Test.insert()
-        self.registry.Test.query().filter_by(id=test.id).update({'col': False})
-        self.assertEqual(test.col, False)
-        self.registry.expire(test, ['col'])
-        self.assertEqual(test.col, '')
-
-    def test_utext(self):
-        registry = self.init_registry(simple_column, ColumnType=uText)
-        test = registry.Test.insert(col='col')
-        self.assertEqual(test.col, 'col')
-
-    def test_utext_with_False(self):
-        registry = self.init_registry(simple_column, ColumnType=uText)
-        test = registry.Test.insert(col=False)
-        self.assertEqual(test.col, False)
-        self.registry.flush()
-        self.registry.expire(test, ['col'])
-        self.assertEqual(test.col, '')
-
-    def test_utext_set_False(self):
-        registry = self.init_registry(simple_column, ColumnType=uText)
-        test = registry.Test.insert()
-        test.col = False
-        self.assertEqual(test.col, False)
-        self.registry.flush()
-        self.registry.expire(test, ['col'])
-        self.assertEqual(test.col, '')
-
-    def test_utext_query_False(self):
-        registry = self.init_registry(simple_column, ColumnType=uText)
-        test = registry.Test.insert()
-        self.registry.Test.query().filter_by(id=test.id).update({'col': False})
-        self.assertEqual(test.col, False)
-        self.registry.expire(test, ['col'])
-        self.assertEqual(test.col, '')
-
     def test_date(self):
         from datetime import date
 
@@ -632,6 +591,51 @@ class TestColumns(DBTestCase):
         registry.flush()
         self.assertIsNotNone(test.update_at)
 
+    def test_datetime_with_default_timezone_tz(self):
+        import datetime
+        import pytz
+
+        timezone = pytz.timezone('Asia/Tokyo')
+        now = datetime.datetime.now()
+        registry = self.init_registry(
+            simple_column, ColumnType=DateTime,
+            default_timezone=timezone)
+        field = registry.loaded_namespaces_first_step['Model.Test']['col']
+        self.assertIs(field.default_timezone, timezone)
+
+        test = registry.Test.insert(col=now)
+        self.assertIs(test.col.tzinfo.zone, timezone.zone)
+
+    def test_datetime_with_default_timezone_str(self):
+        import datetime
+        import pytz
+
+        timezone = pytz.timezone('Asia/Tokyo')
+        now = datetime.datetime.now()
+        registry = self.init_registry(
+            simple_column, ColumnType=DateTime,
+            default_timezone='Asia/Tokyo')
+        field = registry.loaded_namespaces_first_step['Model.Test']['col']
+        self.assertIs(field.default_timezone, timezone)
+
+        test = registry.Test.insert(col=now)
+        self.assertIs(test.col.tzinfo.zone, timezone.zone)
+
+    def test_datetime_with_default_global_timezone_str(self):
+        import datetime
+        import pytz
+
+        timezone = pytz.timezone('Asia/Tokyo')
+        now = datetime.datetime.now()
+        with DBTestCase.Configuration(default_timezone='Asia/Tokyo'):
+            registry = self.init_registry(simple_column, ColumnType=DateTime)
+
+        field = registry.loaded_namespaces_first_step['Model.Test']['col']
+        self.assertIs(field.default_timezone, timezone)
+
+        test = registry.Test.insert(col=now)
+        self.assertIs(test.col.tzinfo.zone, timezone.zone)
+
     def test_interval(self):
         from datetime import timedelta
 
@@ -774,7 +778,9 @@ class TestColumns(DBTestCase):
         Test.insert(col={'a': 'test'})
         Test.insert(col={'b': 'test'})
         self.assertEqual(
-            Test.query().filter(Test.col == {'a': 'test'}).count(), 2)
+            Test.query().filter(
+                Test.col['a'].cast(SA_String) == '"test"').count(),
+            2)
 
     def test_json_null(self):
         registry = self.init_registry(simple_column, ColumnType=Json)
@@ -782,8 +788,8 @@ class TestColumns(DBTestCase):
         Test.insert(col=None)
         Test.insert(col=None)
         Test.insert(col={'a': 'test'})
-        self.assertEqual(Test.query().filter(Test.col == Json.Null).count(), 2)
-        self.assertEqual(Test.query().filter(Test.col != Json.Null).count(), 1)
+        self.assertEqual(Test.query().filter(Test.col.is_(None)).count(), 2)
+        self.assertEqual(Test.query().filter(Test.col.isnot(None)).count(), 1)
 
     def test_add_default_classmethod(self):
         val = 'test'
@@ -991,3 +997,152 @@ class TestColumns(DBTestCase):
         Test = registry.Test
         test = Test.query().filter(Test.col == f).one()
         self.assertEqual(test.col.url, f.url)
+
+    @skipIf(not has_phonenumbers, "phonenumbers is not installed")
+    def test_phonenumbers_at_insert(self):
+        registry = self.init_registry(simple_column, ColumnType=PhoneNumber)
+        test = registry.Test.insert(col='+120012301')
+        self.assertEqual(test.col.national, '20012301')
+        self.assertEqual(
+            registry.execute('Select col from test').fetchone()[0],
+            '+120012301')
+
+    @skipIf(not has_phonenumbers, "phonenumbers is not installed")
+    def test_phonenumbers_at_setter(self):
+        registry = self.init_registry(simple_column, ColumnType=PhoneNumber)
+        test = registry.Test.insert()
+        test.col = '+120012301'
+        self.assertEqual(test.col.national, '20012301')
+        registry.flush()
+        self.assertEqual(
+            registry.execute('Select col from test').fetchone()[0],
+            '+120012301')
+
+    @skipIf(not has_phonenumbers, "phonenumbers is not installed")
+    def test_phonenumbers_obj_at_insert(self):
+        registry = self.init_registry(simple_column, ColumnType=PhoneNumber)
+        col = PN("+120012301", None)
+        test = registry.Test.insert(col=col)
+        self.assertEqual(test.col, col)
+        self.assertEqual(
+            registry.execute('Select col from test').fetchone()[0],
+            '+120012301')
+
+    @skipIf(not has_phonenumbers, "phonenumbers is not installed")
+    def test_phonenumbers_obj_at_setter(self):
+        registry = self.init_registry(simple_column, ColumnType=PhoneNumber)
+        test = registry.Test.insert()
+        test.col = PN("+120012301", None)
+        self.assertEqual(test.col.national, '20012301')
+        registry.flush()
+        self.assertEqual(
+            registry.execute('Select col from test').fetchone()[0],
+            '+120012301')
+
+    def test_email_at_insert(self):
+        registry = self.init_registry(simple_column, ColumnType=Email)
+        test = registry.Test.insert(col='John.Smith@foo.com')
+        self.assertEqual(test.col, 'john.smith@foo.com')
+        self.assertEqual(
+            registry.Test.query().filter_by(col='John.Smith@foo.com').count(),
+            1)
+
+    def test_email_at_setter(self):
+        registry = self.init_registry(simple_column, ColumnType=Email)
+        test = registry.Test.insert()
+        test.col = 'John.Smith@foo.com'
+        self.assertEqual(test.col, 'john.smith@foo.com')
+        self.assertEqual(
+            registry.Test.query().filter_by(col='John.Smith@foo.com').count(),
+            1)
+
+    @skipIf(not has_pycountry, "pycountry is not installed")
+    def test_pycoundtry_at_insert(self):
+        registry = self.init_registry(simple_column, ColumnType=Country)
+        test = registry.Test.insert(col='FR')
+        self.assertIs(test.col, pycountry.countries.get(alpha_2='FR'))
+        self.assertEqual(test.col.name, 'France')
+        self.assertEqual(
+            registry.execute('Select col from test').fetchone()[0], 'FRA')
+
+    @skipIf(not has_pycountry, "pycountry is not installed")
+    def test_pycoundtry_at_insert_with_alpha_3(self):
+        registry = self.init_registry(
+            simple_column, ColumnType=Country, mode='alpha_3')
+        test = registry.Test.insert(col='FRA')
+        self.assertIs(test.col, pycountry.countries.get(alpha_2='FR'))
+        self.assertEqual(test.col.name, 'France')
+        self.assertEqual(
+            registry.execute('Select col from test').fetchone()[0], 'FRA')
+
+    @skipIf(not has_pycountry, "pycountry is not installed")
+    def test_pycoundtry_at_insert_with_object(self):
+        registry = self.init_registry(simple_column, ColumnType=Country)
+        fr = pycountry.countries.get(alpha_2='FR')
+        test = registry.Test.insert(col=fr)
+        self.assertIs(test.col, fr)
+        self.assertEqual(test.col.name, 'France')
+        self.assertEqual(
+            registry.execute('Select col from test').fetchone()[0], 'FRA')
+
+    @skipIf(not has_pycountry, "pycountry is not installed")
+    def test_pycoundtry_at_update(self):
+        registry = self.init_registry(simple_column, ColumnType=Country)
+        test = registry.Test.insert()
+        test.col = 'FR'
+        registry.flush()
+        self.assertIs(test.col, pycountry.countries.get(alpha_2='FR'))
+        self.assertEqual(test.col.name, 'France')
+        self.assertEqual(
+            registry.execute('Select col from test').fetchone()[0], 'FRA')
+
+    @skipIf(not has_pycountry, "pycountry is not installed")
+    def test_pycoundtry_at_update_with_alpha_3(self):
+        registry = self.init_registry(
+            simple_column, ColumnType=Country, mode='alpha_3')
+        test = registry.Test.insert()
+        test.col = 'FRA'
+        registry.flush()
+        self.assertIs(test.col, pycountry.countries.get(alpha_2='FR'))
+        self.assertEqual(test.col.name, 'France')
+        self.assertEqual(
+            registry.execute('Select col from test').fetchone()[0], 'FRA')
+
+    @skipIf(not has_pycountry, "pycountry is not installed")
+    def test_pycoundtry_at_update_with_object(self):
+        registry = self.init_registry(simple_column, ColumnType=Country)
+        fr = pycountry.countries.get(alpha_2='FR')
+        test = registry.Test.insert()
+        test.col = fr
+        registry.flush()
+        self.assertIs(test.col, fr)
+        self.assertEqual(test.col.name, 'France')
+        self.assertEqual(
+            registry.execute('Select col from test').fetchone()[0], 'FRA')
+
+    @skipIf(not has_pycountry, "pycountry is not installed")
+    def test_pycoundtry_query_is_with_object(self):
+        registry = self.init_registry(simple_column, ColumnType=Country)
+        Test = registry.Test
+        fr = pycountry.countries.get(alpha_2='FR')
+        Test.insert(col='FR')
+        self.assertEqual(Test.query().filter(Test.col == fr).count(), 1)
+
+    @skipIf(not has_pycountry, "pycountry is not installed")
+    def test_pycoundtry_query_is_with_alpha_3(self):
+        registry = self.init_registry(simple_column, ColumnType=Country)
+        Test = registry.Test
+        Test.insert(col='FR')
+        self.assertEqual(Test.query().filter(Test.col == 'FRA').count(), 1)
+
+    @skipIf(not has_pycountry, "pycountry is not installed")
+    def test_pycoundtry_query_insert_wrong(self):
+        registry = self.init_registry(simple_column, ColumnType=Country)
+        with self.assertRaises(LookupError):
+            registry.Test.insert(col='WG')
+
+    @skipIf(not has_pycountry, "pycountry is not installed")
+    def test_pycoundtry_query_insert_by_wrong_query(self):
+        registry = self.init_registry(simple_column, ColumnType=Country)
+        with self.assertRaises(Exception):
+            registry.execute("insert into test (col) values ('WG2')")
