@@ -11,20 +11,12 @@ from anyblok.tests.testcase import DBTestCase
 from anyblok import Declarations
 from sqlalchemy.exc import IntegrityError
 from anyblok.field import FieldException
-from anyblok.column import (Integer,
-                            String,
-                            SmallInteger,
-                            BigInteger,
-                            Float,
-                            Decimal,
-                            uString,
-                            Boolean,
-                            DateTime,
-                            Date,
-                            Time,
-                            Sequence)
+from anyblok.column import (
+    Integer, String, BigInteger, Float, Decimal, Boolean, DateTime, Date, Time,
+    Sequence)
 from anyblok.relationship import Many2One
 from sqlalchemy import ForeignKeyConstraint
+from sqlalchemy.engine.reflection import Inspector
 
 
 register = Declarations.register
@@ -228,9 +220,6 @@ class TestMany2One(DBTestCase):
     def test_autodetect_type_integer(self):
         self.check_autodetect_type(Integer)
 
-    def test_autodetect_type_small_integer(self):
-        self.check_autodetect_type(SmallInteger)
-
     def test_autodetect_type_big_integer(self):
         self.check_autodetect_type(BigInteger)
 
@@ -243,9 +232,6 @@ class TestMany2One(DBTestCase):
     def test_autodetect_type_string(self):
         self.check_autodetect_type(String)
 
-    def test_autodetect_type_ustring(self):
-        self.check_autodetect_type(uString)
-
     def test_autodetect_type_boolean(self):
         self.check_autodetect_type(Boolean)
 
@@ -333,6 +319,44 @@ class TestMany2One(DBTestCase):
         with self.assertRaises(IntegrityError):
             registry.Person.insert(name="Other", address=address)
 
+    def test_add_index_constraint(self):
+        def add_in_registry():
+
+            @register(Model)
+            class Address:
+
+                id = Integer(primary_key=True)
+
+            @register(Model)
+            class Person:
+
+                name = String(primary_key=True)
+                address = Many2One(model=Model.Address, index=True)
+
+        registry = self.init_registry(add_in_registry)
+        inspector = Inspector(registry.session.connection())
+        indexes = inspector.get_indexes(registry.Person.__tablename__)
+        self.assertEqual(len(indexes), 1)
+
+    def test_add_primary_keys_constraint(self):
+        def add_in_registry():
+
+            @register(Model)
+            class Address:
+
+                id = Integer(primary_key=True)
+
+            @register(Model)
+            class Person:
+
+                name = String(primary_key=True)
+                address = Many2One(model=Model.Address, primary_key=True)
+
+        registry = self.init_registry(add_in_registry)
+        inspector = Inspector(registry.session.connection())
+        pks = inspector.get_primary_keys(registry.Person.__tablename__)
+        self.assertIn('address_id', pks)
+
     def test_complet_with_multi_foreign_key(self):
 
         def add_in_registry():
